unit sfrmMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, REST.Types, REST.Client,
  Data.Bind.Components, Data.Bind.ObjectScope, Vcl.ExtCtrls, Vcl.StdCtrls,
  Vcl.ExtDlgs,  Vcl.Imaging.JConsts, Vcl.Imaging.jpeg, json, IdBaseComponent,
  IdComponent, IdTCPConnection, IdTCPClient, IdHTTP;

type
  TfrmMain = class(TForm)
    RESTClient: TRESTClient;
    RESTRequest: TRESTRequest;
    RESTResponse: TRESTResponse;
    imgOri: TImage;
    splImages: TSplitter;
    imgModerated: TImage;
    pnlBottom: TPanel;
    btnBrowse: TButton;
    btnModerate: TButton;
    dlgOpenImage: TOpenDialog;
    lblAPI: TLabel;
    edtAPIKey: TEdit;
    lblWait: TLabel;
    procedure FormResize(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnModerateClick(Sender: TObject);
  private
    strImgPath: string;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.btnBrowseClick(Sender: TObject);
var
   Picture: TPicture;
   bmp: TBitmap;
   Jpg: TJPEGImage;
begin
  if not dlgOpenImage.Execute then
    exit;
  strImgPath := dlgOpenImage.FileName;
  btnModerate.Enabled := strImgPath <> '';

  Picture := TPicture.Create;
  Picture.LoadFromFile(strImgPath);
  imgOri.Picture := Picture;

  jpg := TJpegImage.Create;
  jpg.LoadFromFile(strImgPath);
  imgModerated.Picture.Bitmap.Assign(jpg);
  jpg.Free;
end;

procedure TfrmMain.btnModerateClick(Sender: TObject);
var
  lparam : Trestrequestparameter;
  jsonObj: TJSONObject;
  JSonValue  : TJSonValue;
  JSONArray, JSONArrayBound: TJSONArray;
  strJson: string;
  canv: TCanvas;
  i, posTop, posLeft, posWidth, posHeight: Integer;
begin
  lblWait.Visible := true;
  Application.ProcessMessages;
  RESTRequest.Params.Clear;
  RESTResponse.RootElement := '';
  lparam := RESTRequest.Params.AddItem;
  lparam.name := 'api-key';
  lparam.Value := edtAPIKey.Text;
  lparam.ContentType := ctNone;
  lparam.Kind := pkHTTPHEADER;
  lparam.Options := [poDoNotEncode];

  lparam := RESTRequest.Params.AddItem;
  lparam.name := 'image';
  lparam.Value := strImgPath;
  lparam.ContentType := ctNone;
  lparam.Kind := pkFile;
  lparam.Options := [poDoNotEncode];

  RESTRequest.Execute;
  if not RESTResponse.Status.Success then
    showmessage(RESTResponse.StatusText + ' ' +
      inttostr(RESTResponse.StatusCode))
  else
  begin
    canv := imgModerated.Canvas;
    canv.Pen.Color := clRed;
    canv.Pen.Width := 5;

    jsonObj := RESTResponse.JSONValue as TJSONObject;
    JSonValue := jsonObj.Get('output').JsonValue;
    JSONArray := JSonValue.GetValue<TJSONArray>('detections');

    for I := 0 to JSONArray.Count -1 do
    begin
      JSONArrayBound := JSONArray.Items[I].GetValue<TJSONArray>('bounding_box');
      posTop :=  strtoint(JSONArrayBound.Items[1].Value);
      posLeft :=  strtoint(JSONArrayBound.Items[0].Value);
      posWidth :=  strtoint(JSONArrayBound.Items[3].Value);
      posHeight :=  strtoint(JSONArrayBound.Items[2].Value);

      canv.MoveTo(posLeft,posTop);
      canv.LineTo(posLeft + posWidth, posTop);

      canv.MoveTo(posLeft + posWidth, posTop);
      canv.LineTo(posLeft + posWidth, posTop + posHeight);

      canv.MoveTo(posLeft + posWidth, posTop + posHeight);
      canv.LineTo(posLeft, posTop + posHeight);

      canv.MoveTo(posLeft, posTop + posHeight);
      canv.LineTo(posLeft,posTop);

      canv.TextOut(posLeft,posTop, JSONArray.Items[I].GetValue<string>('name'));
    end;
  end;
  lblWait.Visible := false;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  TPicture.RegisterFileFormat('jpg', sJPEGImageFile, TJPEGImage);
  TPicture.RegisterFileFormat('jpeg', sJPEGImageFile, TJPEGImage);
  TPicture.RegisterFileFormat('png', sJPEGImageFile, TJPEGImage);
  TPicture.RegisterFileFormat('bmp', sJPEGImageFile, TJPEGImage);
end;

procedure TfrmMain.FormResize(Sender: TObject);
begin
  imgOri.Width := round(self.ClientWidth / 2);
end;

end.
